Package: PSS.Health
Type: Package
Title: Power and Sample Size for Health Researchers via Shiny
Depends: R (>= 4.1.0)
Version: 1.1.5
Date: 2025-11-23
Language: pt
Authors@R: c(person(given = "Rogério",
                    family = "Boff Borges",
                    role = c("aut", "cre"),
                    email = "roborges@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0002-2548-1889")),
          person(given = "Guilherme",
                    family = "Serpa Azambuja",
                    role = c("aut"),
                    email = "gsazambuja@gmail.com",
                    comment = c(ORCID = "orcid.org/0000-0001-6246-4538")),
          person(given = "Aline",
                    family = "Castello Branco Mancuso",
                    role = c("aut"),
                    email = "amancuso@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0001-6033-8335")),
          person(given = "Vanessa",
                    family = "Bielefeldt Leotti",
                    role = c("aut"),
                    email = "vleotti@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0003-3860-9367")),
          person(given = "Vânia",
                    family = "Naomi Hirakata",
                    role = c("aut"),
                    email = "vhirakata@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0003-4645-2080")),
          person(given = "Suzi",
                    family = "Alves Camey",
                    role = c("aut"),
                    email = "scamey@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0002-5564-081X")),
          person(given = "Stela",
                    family = "Maris de Jezus Castro",
                    role = c("aut"),
                    email = "stelacastro@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0001-5862-6709")),
          person(given = "Gustavo",
                    family = "Thomas",
                    role = c("aut"),
                    email = "gthomas@hcpa.edu.br",
                    comment = c(ORCID = "orcid.org/0000-0002-4327-8307")),
          person(given = "Hospital de Clínicas de Porto Alegre",
                    role = c("fnd"))
          )
Description: Power and Sample Size for Health Researchers is a Shiny application that brings together a 
    series of functions  related to sample size and power calculations for common analysis in the healthcare 
    field. There are functionalities to calculate the power, sample size to estimate or test hypotheses for means and 
    proportions (including test for correlated groups, equivalence, non-inferiority and superiority), association, correlations coefficients, 
    regression coefficients (linear, logistic, gamma, and Cox), linear mixed model, 
    Cronbach's alpha, interobserver agreement, intraclass correlation coefficients,
    limit of agreement on Bland-Altman plots,
    area under the curve, sensitivity and specificity incorporating the prevalence of disease. 
    You can also use the online version at <https://hcpa-unidade-bioestatistica.shinyapps.io/PSS_Health/>.
URL: https://hcpa-unidade-bioestatistica.shinyapps.io/PSS_Health/
Imports: dplyr, DT, easypower, EnvStats, epiR, ggplot2,
        ICC.Sample.Size, kappaSize, longpower, plotly, powerMediation,
        powerSurvEpi, presize, pROC, pwr, pwr2, shiny, shinycssloaders,
        shinyFeedback, shinyhelper, writexl
License: GPL (>= 2)
Encoding: UTF-8
Contact: 'mailto:psshealth@hcpa.edu.br'
Suggests: rmarkdown, knitr, MESS, WebPower
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-11-23 20:31:26 UTC; roger
Author: Rogério Boff Borges [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2548-1889>),
  Guilherme Serpa Azambuja [aut] (ORCID:
    <https://orcid.org/0000-0001-6246-4538>),
  Aline Castello Branco Mancuso [aut] (ORCID:
    <https://orcid.org/0000-0001-6033-8335>),
  Vanessa Bielefeldt Leotti [aut] (ORCID:
    <https://orcid.org/0000-0003-3860-9367>),
  Vânia Naomi Hirakata [aut] (ORCID:
    <https://orcid.org/0000-0003-4645-2080>),
  Suzi Alves Camey [aut] (ORCID: <https://orcid.org/0000-0002-5564-081X>),
  Stela Maris de Jezus Castro [aut] (ORCID:
    <https://orcid.org/0000-0001-5862-6709>),
  Gustavo Thomas [aut] (ORCID: <https://orcid.org/0000-0002-4327-8307>),
  Hospital de Clínicas de Porto Alegre [fnd]
Maintainer: Rogério Boff Borges <roborges@hcpa.edu.br>
Repository: CRAN
Date/Publication: 2025-11-23 21:10:02 UTC
Built: R 4.6.0; ; 2026-04-30 15:24:41 UTC; unix
