---
title: "PSS Health publications"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{PSS Health}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```



See below the published articles that can help you use PSS Health:
<br>
<br>

<ul>
<li><b><a href="https://doi.org/10.22491/2357-9730.109542" target="_blank">Power and Sample Size for Health Researchers: uma ferramenta para cálculo de tamanho amostral e poder do teste voltado a pesquisadores da área da saúde</a></b></li>
<br>
<li><b><a href="https://doi.org/10.22491/2357-9730.112466" target="_blank">PSS Health: como calcular tamanho de amostra para estimar média, proporção e correlação</a></b></li>
<br>
<li><b><a href="https://doi.org/10.22491/2357-9730.120997" target="_blank">PSS Health: como calcular tamanho de amostra para testes de comparação de médias de dois grupos</a></b></li>
<br>
<li><b><a href="https://doi.org/10.22491/2357-9730.126843" target="_blank">PSS Health: como calcular tamanho de amostra para testar relações de variáveis com um desfecho binário</a></b></li>
<br>
<li><b><a href="https://doi.org/10.22491/2357-9730.139152" target="_blank">PSS Health: como calcular tamanho de amostra para testar hipóteses de variáveis quantitativas</a></b></li>
</ul>


