---
title: "Medida de efeito f"
author: "Unidade de Bioestatística"
date: "07/11/2020"
output: html_document
---

O tamanho de efeito $f$ é uma medida de efeito utilizada em análise de variância. Sua fórmula é dada por

$$
f = \sqrt{\dfrac{SQ_{entre}}{SQ_{dentro}}},
$$

onde $SQ_{entre}$ é a soma de quadrados entre os grupos e $SQ_{dentro}$ é a soma de quadrados dentro dos grupos. 
  

&nbsp;
&nbsp;


O $f$ também pode ser obtido da medida de efeito $\eta^2$ pela fórmula

$$
f = \sqrt{\dfrac{\eta^2}{(1-\eta^2)}}.
$$


<br><br><b>Pode ser um valor da literatura ou um valor que o pesquisador deseja encontrar e que tenha relevância clínica.</b>
