---
title: "Macnemar"
author: "Unidade de Bioestatistica"
date: "15/02/2021"
output: 
  html_document: 
    df_print: kable
---


**Objetivo do estudo:** comparar duas amostras pareadas quanto ao resultado de uma variável dicotômica. É muito utilizado para analisar dados oriundos de planejamentos do tipo 'antes' e 'depois' onde o mesmo sujeito é avaliado quanto a uma informação binária antes e depois de uma determinada interferência. Neste tipo de análise, os dados podem ser resumidos em uma tabela de contingência do tipo 2X2, conforme abaixo:


<!-- |**Antes/Depois**     |*Sim*   | *Não*| -->
<!-- |------ | ------ | ------- -->
<!-- |***Antes***  | $p_a$      | $p_c$| -->
<!-- |***Depois*** | $p_b$      | $p_d$| -->


&nbsp; &nbsp;  &nbsp; &nbsp; &nbsp; &nbsp;  &nbsp; &nbsp;  &nbsp; &nbsp; &nbsp; **Depois**

**Antes** &nbsp; &nbsp; &nbsp; &nbsp; ***Sim*** &nbsp; &nbsp; ***Não***

***Sim*** &nbsp; &nbsp;  &nbsp; &nbsp; &nbsp; &nbsp;  &nbsp;  $p_a$ &nbsp; &nbsp; &nbsp; $p_b$
 
***Não*** &nbsp; &nbsp;  &nbsp; &nbsp; &nbsp; &nbsp;  &nbsp;  $p_c$ &nbsp; &nbsp; &nbsp; $p_d$



onde as células $p_b$ e $p_c$ representam o percentual do total de pares discordantes (aqueles que obtiveram respostas diferentes). Mais detalhes sobre o uso dessa aba em **[PSS Health: como calcular tamanho de amostra para testar relações de variáveis com um desfecho binário](https://doi.org/10.22491/2357-9730.126843)**.


